@@ -13,6 +13,7 @@
 #    See the License for the specific language governing permissions and
 #    limitations under the License.
 
+import datetime
 import hashlib
 import json
 import sys
@@ -33,6 +34,7 @@ from mistral import utils
 
 # Definition objects.
 
+CONF = cfg.CONF
 LOG = logging.getLogger(__name__)
 
 
@@ -111,9 +113,14 @@ class Workbook(Definition):
     """Contains info about workbook (including definition in Mistral DSL)."""
 
     __tablename__ = 'workbooks_v2'
+    namespace = sa.Column(sa.String(255), nullable=True)
 
     __table_args__ = (
-        sa.UniqueConstraint('name', 'project_id'),
+        sa.UniqueConstraint(
+            'name',
+            'namespace',
+            'project_id'
+        ),
         sa.Index('%s_project_id' % __tablename__, 'project_id'),
         sa.Index('%s_scope' % __tablename__, 'scope'),
     )
@@ -195,6 +202,13 @@ class ActionExecution(Execution):
     accepted = sa.Column(sa.Boolean(), default=False)
     input = sa.Column(st.JsonLongDictType(), nullable=True)
     output = sa.orm.deferred(sa.Column(st.JsonLongDictType(), nullable=True))
+    last_heartbeat = sa.Column(
+        sa.DateTime,
+        default=lambda: utils.utc_now_sec() + datetime.timedelta(
+            seconds=CONF.action_heartbeat.first_heartbeat_timeout
+        )
+    )
+    is_sync = sa.Column(sa.Boolean(), default=None, nullable=True)
 
 
 class WorkflowExecution(Execution):
@@ -239,7 +253,7 @@ class TaskExecution(Execution):
 
     # Main properties.
     action_spec = sa.Column(st.JsonLongDictType())
-    unique_key = sa.Column(sa.String(250), nullable=True)
+    unique_key = sa.Column(sa.String(255), nullable=True)
     type = sa.Column(sa.String(10))
 
     # Whether the task is fully processed (publishing and calculating commands
@@ -370,7 +384,7 @@ class DelayedCall(mb.MistralModelBase):
     method_arguments = sa.Column(st.JsonDictType())
     serializers = sa.Column(st.JsonDictType())
     key = sa.Column(sa.String(250), nullable=True)
-    auth_context = sa.Column(st.JsonDictType())
+    auth_context = sa.Column(st.JsonMediumDictType())
     execution_time = sa.Column(sa.DateTime, nullable=False)
     processing = sa.Column(sa.Boolean, default=False, nullable=False)
 
@@ -555,7 +569,7 @@ class NamedLock(mb.MistralModelBase):
     sa.UniqueConstraint('name')
 
     id = mb.id_column()
-    name = sa.Column(sa.String(250))
+    name = sa.Column(sa.String(255))
 
 
 sa.UniqueConstraint(NamedLock.name)
