@@ -57,8 +57,7 @@ def _process_queue(queue):
 
     for operation, args in queue:
         if operation == _RUN_ACTION:
-            action_ex, action_def, target, execution_context, \
-                timeout = args
+            action_ex, action_def, target, execution_context, timeout = args
 
             executor.run_action(
                 action_ex.id,
@@ -100,7 +99,7 @@ def process(func):
             # NOTE(rakhmerov): Since we make RPC calls to the engine itself
             # we need to process the action queue asynchronously in a new
             # thread. Otherwise, if we have one engine process the engine
-            # will may send a request to itself while already processing
+            # may send a request to itself while already processing
             # another one. In conjunction with blocking RPC it will lead
             # to a deadlock (and RPC timeout).
             def _within_new_thread():
