@@ -16,8 +16,10 @@ from oslo_config import cfg
 import testtools
 
 from mistral.db.v2 import api as db_api
+from mistral.lang.v2 import tasks as tasks_lang
 from mistral.services import workflows as wf_service
 from mistral.tests.unit.engine import base
+from mistral import utils
 from mistral.workflow import states
 
 
@@ -871,18 +873,11 @@ class JoinEngineTest(base.EngineTestCase):
             state=states.WAITING
         )
 
-        calls = db_api.get_delayed_calls()
-
-        mtd_name = 'mistral.engine.task_handler._refresh_task_state'
-
-        cnt = sum([1 for c in calls if c.target_method_name == mtd_name])
-
-        # There can be 2 calls with different value of 'processing' flag.
-        self.assertTrue(cnt == 1 or cnt == 2)
-
         # Stop the workflow.
         self.engine.stop_workflow(wf_ex.id, state=states.CANCELLED)
 
+        mtd_name = 'mistral.engine.task_handler._refresh_task_state'
+
         self._await(
             lambda:
             len(db_api.get_delayed_calls(target_method_name=mtd_name)) == 0
@@ -929,18 +924,11 @@ class JoinEngineTest(base.EngineTestCase):
             state=states.WAITING
         )
 
-        calls = db_api.get_delayed_calls()
-
-        mtd_name = 'mistral.engine.task_handler._refresh_task_state'
-
-        cnt = sum([1 for c in calls if c.target_method_name == mtd_name])
-
-        # There can be 2 calls with different value of 'processing' flag.
-        self.assertTrue(cnt == 1 or cnt == 2)
-
         # Stop the workflow.
         db_api.delete_workflow_execution(wf_ex.id)
 
+        mtd_name = 'mistral.engine.task_handler._refresh_task_state'
+
         self._await(
             lambda:
             len(db_api.get_delayed_calls(target_method_name=mtd_name)) == 0
@@ -1269,3 +1257,40 @@ class JoinEngineTest(base.EngineTestCase):
 
                 self.assertEqual(task.name, task_result,
                                  "The result of task must equal own name")
+
+    def test_join_with_long_name(self):
+        long_task_name = utils.generate_string(
+            tasks_lang.MAX_LENGTH_JOIN_TASK_NAME
+        )
+        wf_text = """---
+        version: '2.0'
+
+        wf:
+          tasks:
+            task1:
+              on-success:
+                - {0}
+
+            task2:
+              on-success:
+                - {0}
+
+            {0}:
+              join: all
+        """.format(long_task_name)
+
+        wf_service.create_workflows(wf_text)
+        wf_ex = self.engine.start_workflow('wf')
+        self.await_workflow_success(wf_ex.id)
+
+        with db_api.transaction():
+            wf_ex = db_api.get_workflow_execution(wf_ex.id)
+            tasks = wf_ex.task_executions
+
+        task1 = self._assert_single_item(tasks, name='task1')
+        task2 = self._assert_single_item(tasks, name='task2')
+        task3 = self._assert_single_item(tasks, name=long_task_name)
+
+        self.assertEqual(states.SUCCESS, task1.state)
+        self.assertEqual(states.SUCCESS, task2.state)
+        self.assertEqual(states.SUCCESS, task3.state)
